package usercontract

import (
	"testing"

	"github.com/golang/mock/gomock"

	contractapi "git.huawei.com/poissonsearch/wienerchain/contract/docker-container/contract-go/contractapi"
)

var (
	key, value = "a", "huaweichain"

	startKey, endKey = "a", "c"
)

func TestInsert(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()
	// 打桩，指定所调用方法中使用到的接口的入参和返回值
	stub := contractapi.NewMockContractStub(ctrl)
	stub.EXPECT().FuncName().Return("insert")
	stub.EXPECT().Parameters().Return([][]byte{[]byte(key), []byte(value)})
	stub.EXPECT().PutKV(key, []byte(value)).Return(nil)
	// 方法调用
	invoke(t, stub)
}

func TestQuery(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()
	// 打桩，指定所调用方法中使用到的接口的入参和返回值
	stub := contractapi.NewMockContractStub(ctrl)
	stub.EXPECT().FuncName().Return("query")
	stub.EXPECT().Parameters().Return([][]byte{[]byte(key)})
	stub.EXPECT().GetKV(key).Return([]byte(value), nil)
	// 方法调用
	invoke(t, stub)
}

func TestDelete(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()
	// 打桩，指定所调用方法中使用到的接口的入参和返回值
	stub := contractapi.NewMockContractStub(ctrl)
	stub.EXPECT().FuncName().Return("delete")
	stub.EXPECT().Parameters().Return([][]byte{[]byte(key)})
	stub.EXPECT().GetKV(key).Return([]byte(value), nil)
	stub.EXPECT().DelKV(key).Return(nil)
	// 方法调用
	invoke(t, stub)
}

func TestQueryKeyHistory(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()
	// 打桩，指定所调用方法中使用到的接口的入参和返回值
	historyIterator := contractapi.NewMockHistoryIterator(ctrl)
	historyIterator.EXPECT().Next().Return(true)
	historyIterator.EXPECT().Value().Return([]byte(value))
	historyIterator.EXPECT().Close().Return()
	historyIterator.EXPECT().Version().Return(uint64(2), int32(0))
	historyIterator.EXPECT().TxHash().Return([]byte("xxxxxxxxxxxxxxxxxxxxxx"))
	historyIterator.EXPECT().IsDeleted().Return(false)
	historyIterator.EXPECT().Timestamp().Return(uint64(1636440016000000))
	historyIterator.EXPECT().Next().Return(false)

	stub := contractapi.NewMockContractStub(ctrl)
	stub.EXPECT().FuncName().Return("queryKeyHistory")
	stub.EXPECT().Parameters().Return([][]byte{[]byte(key)})
	stub.EXPECT().GetKeyHistoryIterator(startKey).Return(historyIterator, nil)
	// 方法调用
	invoke(t, stub)
}

func TestQueryInRange(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()
	// 打桩，指定所调用方法中使用到的接口的入参和返回值
	iterator := contractapi.NewMockIterator(ctrl)
	iterator.EXPECT().Next().Return(true)
	iterator.EXPECT().Key().Return(key)
	iterator.EXPECT().Value().Return([]byte(value))
	iterator.EXPECT().Close().Return()
	iterator.EXPECT().Next().Return(false)

	stub := contractapi.NewMockContractStub(ctrl)
	stub.EXPECT().FuncName().Return("queryInRange")
	stub.EXPECT().Parameters().Return([][]byte{[]byte(startKey), []byte(endKey)})
	stub.EXPECT().GetIterator(startKey, endKey).Return(iterator, nil)
	// 方法调用
	invoke(t, stub)
}

func TestQueryInRangeMatchCondition(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()
	// 打桩，指定所调用方法中使用到的接口的入参和返回值
	iterator := contractapi.NewMockIterator(ctrl)
	iterator.EXPECT().Next().Return(true)
	iterator.EXPECT().Key().Return(key)
	iterator.EXPECT().Value().Return([]byte(value))
	iterator.EXPECT().Close().Return()
	iterator.EXPECT().Next().Return(false)

	stub := contractapi.NewMockContractStub(ctrl)
	stub.EXPECT().FuncName().Return("queryInRangeMatchCondition")
	stub.EXPECT().Parameters().Return([][]byte{[]byte(startKey), []byte(endKey), []byte("="), []byte(value)})
	stub.EXPECT().GetIterator(startKey, endKey).Return(iterator, nil)
	// 方法调用
	invoke(t, stub)
}

func invoke(t *testing.T, stub *contractapi.MockContractStub) {
	var contract = NewSmartContract()
	invokeRes, err := contract.Invoke(stub)
	if err != nil {
		t.Fatalf("invoke failed, err is: %v", err)
	}
	t.Logf("invoke success: %s\n", string(invokeRes))
}
